"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var cockpit_notifier_1 = require("../agent-events/cockpit-notifier");
var agent_events_conracts_1 = require("../agent-events/agent-events-conracts");
var events = require("events");
var buffer_size_helper_1 = require("./buffer-size-helper");
/**
 * Holds the footprints hits data and builds the DTO for footprints packet
 */
var FootprintsBuffer = /** @class */ (function (_super) {
    __extends(FootprintsBuffer, _super);
    function FootprintsBuffer(agentInstanceData, agentConfig) {
        var _this = _super.call(this) || this;
        _this.formatVersion = "6.0";
        _this.agentInstanceData = agentInstanceData;
        _this._agentConfig = agentConfig;
        _this.currentBufferSize = 0;
        _this.resetState();
        _this.meta = {
            agentId: _this.agentInstanceData.agentId,
            intervals: {
                timedFootprintsCollectionIntervalSeconds: _this._agentConfig.footprintsCollectIntervalSecs.value
            },
            labId: _this._agentConfig.labId.value
        };
        return _this;
    }
    FootprintsBuffer.prototype.addHit = function (footprints, collectionInterval, executionId, testName, isInitFootprints, isFinalFootprints) {
        cockpit_notifier_1.CockpitNotifier.sendEventOnce(agent_events_conracts_1.AgentEventCode.FIRST_TIME_COLLECTED_FP);
        var methodIndices = this.addOrGetIndices(footprints.methods, true);
        var branchIndices = this.addOrGetIndices(footprints.branches, false);
        var executionIdx = this.addOrGetExecutionIndex(executionId);
        this.executions[executionIdx].hits.push({
            branches: branchIndices,
            methods: methodIndices,
            start: collectionInterval.start,
            end: collectionInterval.end,
            testName: testName,
            isInitFootprints: isInitFootprints,
            isFinalFootprints: isFinalFootprints
        });
        this.bufferSizeHelper.incrementSizeCounters(testName, methodIndices.length, branchIndices.length);
        this.verifyBufferSize();
    };
    FootprintsBuffer.prototype.createPacket = function () {
        var packet = this.toJson();
        this.resetState();
        return packet;
    };
    FootprintsBuffer.prototype.resetState = function () {
        this.branches = [];
        this.methods = [];
        this.executions = [];
        this.methodIdToIndex = new Map();
        this.branchIdToIndex = new Map();
        this.bufferSizeHelper = new buffer_size_helper_1.BufferSizeHelper();
    };
    FootprintsBuffer.prototype.hasHitsInBuffer = function () {
        return this.executions.length > 0;
    };
    Object.defineProperty(FootprintsBuffer.prototype, "agentConfig", {
        get: function () {
            return this._agentConfig;
        },
        set: function (value) {
            this._agentConfig = value;
        },
        enumerable: true,
        configurable: true
    });
    FootprintsBuffer.prototype.toJson = function () {
        if (!this.executions || !this.executions.length) {
            return null;
        }
        return {
            formatVersion: this.formatVersion,
            branches: this.branches,
            executions: this.executions,
            meta: this.meta,
            methods: this.methods,
            moduleName: this.moduleName
        };
    };
    FootprintsBuffer.prototype.addOrGetIndices = function (elementIds, isMethods) {
        var _this = this;
        var map = isMethods ? this.methodIdToIndex : this.branchIdToIndex;
        var arr = isMethods ? this.methods : this.branches;
        var indices = [];
        elementIds.forEach(function (id) {
            if (!map.get(id)) {
                _this.bufferSizeHelper.uniqueIdsLengthSum += id.length;
                _this.bufferSizeHelper.uniqueIdsCount++;
                var position = arr.push(id);
                map.set(id, position - 1);
            }
            indices.push(map.get(id));
        });
        return indices;
    };
    ;
    FootprintsBuffer.prototype.addOrGetExecutionIndex = function (executionId) {
        var index = -1;
        this.executions.every(function (execution, idx) {
            if (executionId == execution.executionId) {
                index = idx;
                return false;
            }
            return true;
        });
        return index != -1 ? index : this.executions.push({ executionId: executionId, hits: [] }) - 1;
    };
    FootprintsBuffer.prototype.verifyBufferSize = function () {
        if (this.bufferSizeHelper.calculateBufferSize() >= this._agentConfig.footprintsBufferThresholdMb.value) {
            this.emit(FootprintsBuffer.BUFFER_FULL);
        }
    };
    FootprintsBuffer.BUFFER_FULL = "bufferFull";
    return FootprintsBuffer;
}(events.EventEmitter));
exports.FootprintsBuffer = FootprintsBuffer;
//# sourceMappingURL=footprints-buffer.js.map