"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var HitsConverter = /** @class */ (function () {
    function HitsConverter(relativePathResolver, sourceMapData, projectRoot, logger) {
        this.relativePathResolver = relativePathResolver;
        this.sourceMapData = sourceMapData;
        this.projectRoot = projectRoot;
        this.logger = logger;
    }
    HitsConverter.prototype.convertHits = function (hitFilesData) {
        var _this = this;
        var methodUniqueIds = [];
        var branchUniqueIds = [];
        hitFilesData.forEach(function (fileHit) {
            var relativePath = fileHit.filename;
            if (fileHit.shouldResolveRelativePath) {
                relativePath = _this.relativePathResolver.getRelativePath(fileHit.filename);
            }
            methodUniqueIds = methodUniqueIds.concat(_this.createMethodIds(fileHit.hitMethods, relativePath, fileHit.filename));
            branchUniqueIds = branchUniqueIds.concat(_this.createBranchIds(fileHit.hitBranches, relativePath, fileHit.filename));
        });
        return {
            branches: branchUniqueIds,
            methods: methodUniqueIds
        };
    };
    HitsConverter.prototype.createMethodIds = function (hitFunctions, relativePath, absolutePath) {
        var _this = this;
        var methodIds = [];
        hitFunctions.forEach(function (hit) {
            methodIds.push(_this.getMethodUniqueId(hit.loc.start, relativePath, absolutePath));
            if (hit.decl && hit.decl.start) {
                methodIds.push(_this.getMethodUniqueId(hit.decl.start, relativePath, absolutePath));
            }
        });
        return methodIds;
    };
    HitsConverter.prototype.createBranchIds = function (hitFunctions, relativePath, absolutePath) {
        var _this = this;
        var branchIds = [];
        hitFunctions.forEach(function (hit) {
            hit.hitLeaves.forEach(function (leaveIdx) {
                branchIds.push(_this.getBranchUniqueId(hit.branchData.locations[leaveIdx].start, relativePath, absolutePath, leaveIdx));
            });
        });
        return branchIds;
    };
    HitsConverter.prototype.getMethodUniqueId = function (startLoc, relativePath, absolutePath) {
        var idParts = this.resolveIdParts(startLoc, relativePath, absolutePath);
        return this.buildMethodUniqueId(idParts);
    };
    HitsConverter.prototype.getBranchUniqueId = function (startLoc, relativePath, absolutePath, leaveIndex) {
        var parts = this.resolveIdParts(startLoc, relativePath, absolutePath);
        return parts.path + HitsConverter.BRANCH_ID_DEL + this.formatLoc(parts.start) + HitsConverter.BRANCH_ID_DEL + leaveIndex;
    };
    HitsConverter.prototype.resolveIdParts = function (startLoc, relativePath, absolutePath) {
        var sourcePosition = this.sourceMapData.getSourcePosition(relativePath, absolutePath, startLoc, this.projectRoot);
        return sourcePosition ? sourcePosition : { path: relativePath, start: startLoc };
    };
    HitsConverter.prototype.buildMethodUniqueId = function (parts) {
        return parts.path + HitsConverter.METHOD_ID_DEL + this.formatLoc(parts.start);
    };
    HitsConverter.prototype.formatLoc = function (loc) {
        return loc.line + ',' + loc.column;
    };
    HitsConverter.METHOD_ID_DEL = "@";
    HitsConverter.BRANCH_ID_DEL = "|";
    return HitsConverter;
}());
exports.HitsConverter = HitsConverter;
//# sourceMappingURL=hits-converter.js.map