"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var footprints_buffer_1 = require("./footprints-buffer");
var collection_interval_1 = require("../footprints-process/collection-interval");
var state_tracker_1 = require("../state-tracker");
var cockpit_notifier_1 = require("../agent-events/cockpit-notifier");
var agent_events_conracts_1 = require("../agent-events/agent-events-conracts");
var FootprintsProcess = /** @class */ (function () {
    function FootprintsProcess(cfg, sendToServerWatchdog, keepaliveWatchdog, logger, hitsCollector, hitsConverter, footprintsBuffer, backendProxy, stateTracker) {
        this.isRunning = false;
        this.ongoingRequestsCounter = 0;
        this.footprintsEnqueueOnce = false;
        this.cfg = cfg;
        this.sendToServerWatchdog = sendToServerWatchdog;
        this.keepaliveWatchdog = keepaliveWatchdog;
        this.footprintsBuffer = footprintsBuffer;
        this.hitsConverter = hitsConverter;
        this.hitsCollector = hitsCollector;
        this.backendProxy = backendProxy;
        this.collectionInterval = new collection_interval_1.CollectionInterval(cfg.interval.value);
        this.stateTracker = stateTracker;
        this.logger = logger;
        this.delegateEvents();
    }
    FootprintsProcess.prototype.enqueueCurrentFootprints = function (executionId, testName, executionBsid, isFinalFootprints) {
        if (isFinalFootprints === void 0) { isFinalFootprints = false; }
        this.currentExecutionBsid = executionBsid;
        this.collectionInterval.next();
        var hitFilesData = this.hitsCollector.getHitElements();
        if (hitFilesData.length) {
            this.logger.info("Collecting hits for '" + hitFilesData.length + "' files");
            var convertedHits = this.hitsConverter.convertHits(hitFilesData);
            this.footprintsBuffer.addHit(convertedHits, this.collectionInterval.toJson(), executionId, testName, this.isInitFootprints(), isFinalFootprints);
        }
        else {
            this.logger.info('No files were hits, not collecting footprints');
        }
        if (this.isRunning && this.cfg.sendFootprints.value && this.cfg.enabled.value) {
            this.ensureKeepaliveThreadRunning();
            this.sendToServerWatchdog.start();
        }
        this.footprintsEnqueueOnce = true;
    };
    FootprintsProcess.prototype.updateConfig = function (updatedCfg) {
        this.cfg = updatedCfg;
        if (updatedCfg.sendFootprints.value === false || updatedCfg.enabled.value === false) {
            cockpit_notifier_1.CockpitNotifier.sendEvent(agent_events_conracts_1.AgentEventCode.AGENT_MUTED);
            this.footprintsBuffer.resetState();
            this.stop(function () {
            });
        }
        this.sendToServerWatchdog.setInterval(this.cfg.footprintsSendIntervalSecs.value);
        this.footprintsBuffer.agentConfig = updatedCfg;
    };
    FootprintsProcess.prototype.ensureKeepaliveThreadRunning = function () {
        this.keepaliveWatchdog.start();
    };
    FootprintsProcess.prototype.submitQueuedFootprints = function () {
        return __awaiter(this, void 0, void 0, function () {
            var packet, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.shouldSubmitFootprints()) {
                            return [2 /*return*/];
                        }
                        packet = this.footprintsBuffer.createPacket();
                        if (!packet) return [3 /*break*/, 6];
                        this.ongoingRequestsCounter++;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, 4, 5]);
                        return [4 /*yield*/, this.backendProxy.submitFootprintsV6(packet, this.currentExecutionBsid, this.stateTracker.getTestStage(), this.cfg.buildSessionId.value)];
                    case 2:
                        _a.sent();
                        this.logger.info("Footprints packet submitted successfully. packet contains " + packet.methods.length + " methods, " + packet.branches.length + " branches in " + packet.executions.length + " executions");
                        return [3 /*break*/, 5];
                    case 3:
                        e_1 = _a.sent();
                        this.logger.error("Error while submitting footprints '" + e_1 + "'");
                        cockpit_notifier_1.CockpitNotifier.sendEvent(agent_events_conracts_1.AgentEventCode.FOOTPRINTS_LOSS);
                        return [3 /*break*/, 5];
                    case 4:
                        this.ongoingRequestsCounter--;
                        return [7 /*endfinally*/];
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        this.logger.info('No hits collected nothing to submit');
                        _a.label = 7;
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    FootprintsProcess.prototype.shouldSubmitFootprints = function () {
        if (!this.isRunning) {
            this.logger.info("Agent is not running, not sending footprints");
            return false;
        }
        if (this.cfg.sendFootprints.value === false) {
            this.logger.info("Not sending footprints since agent is configured to not send.");
            return false;
        }
        return true;
    };
    FootprintsProcess.prototype.start = function () {
        if (this.isRunning)
            return;
        if (this.cfg.enabled.value == false)
            return;
        this.isRunning = true;
        this.sendToServerWatchdog.start();
        if (this.footprintsBuffer.hasHitsInBuffer()) {
            this.ensureKeepaliveThreadRunning();
        }
    };
    // TODO: Make async
    FootprintsProcess.prototype.stop = function (callback) {
        var _this = this;
        this.sendToServerWatchdog.stop();
        if (!this.shouldSubmitFootprints()) {
            this.isRunning = false;
            return callback();
        }
        this.isRunning = false;
        this.flushCurrentFootprints(true);
        var packet = this.footprintsBuffer.createPacket();
        if (!packet) {
            this.logger.info('No hits collected, nothing to submit');
            return callback();
        }
        this.logger.debug("Start submitting footprints, triggered by stop event.");
        this.backendProxy.submitFootprintsV6(packet, this.currentExecutionBsid, this.stateTracker.getTestStage(), this.cfg.buildSessionId.value)
            .then(callback)
            .catch(function (err) {
            _this.logger.error(err);
            return callback();
        });
    };
    FootprintsProcess.prototype.handleTestIdChanged = function (newTestIdentifier, previousTestIdentifier) {
        if (previousTestIdentifier == null) {
            this.logger.info("Test identifier changed, previous identifier wasn't set, meaning that we didn't have active test. Skip enqueuing footprints process.");
        }
        else {
            if (!this.stateTracker.shouldCollectHits()) {
                this.logger.info("Test identifier changed, couldn't find active execution for anonymous footprints. Skip enqueuing footprints process.");
                return;
            }
            this.logger.debug("Test identifier changed, start enqueuing footprints process.");
            var prevTestIdentifierParts = state_tracker_1.StateTracker.splitTestIdToExecutionAndTestName(previousTestIdentifier);
            this.enqueueCurrentFootprints(this.stateTracker.currentExecution.executionId, prevTestIdentifierParts.testName, this.stateTracker.currentExecution.buildSessionId);
        }
    };
    FootprintsProcess.prototype.delegateEvents = function () {
        var _this = this;
        this.sendToServerWatchdog.on(FootprintsProcess.ALARM_FIRED, function () {
            _this.logger.debug("Start submitting footprints, triggered by send to server watchdog.");
            _this.submitQueuedFootprints();
        });
        this.keepaliveWatchdog.on(FootprintsProcess.ALARM_FIRED, function () {
            if (!_this.hasOngoingRequests() && !_this.footprintsBuffer.hasHitsInBuffer()) {
                _this.keepaliveWatchdog.stop();
            }
        });
        this.footprintsBuffer.on(footprints_buffer_1.FootprintsBuffer.BUFFER_FULL, function () {
            _this.submitQueuedFootprints();
        });
        this.stateTracker.on(state_tracker_1.StateTracker.EXECUTION_ENDS, function (prevExecution) {
            _this.logger.warn("Execution status has changed to '" + state_tracker_1.StateTracker.EXECUTION_STATUS_PENDING_DELETE + "'. Collecting previous footprints");
            _this.enqueueAndSubmit(prevExecution, true);
        });
        this.stateTracker.on(state_tracker_1.StateTracker.EXECUTION_BSID_CHANGED, function (prevExecution, newBuildSession) {
            _this.logger.warn("Execution points to different build session '" + newBuildSession + "'. Collecting and submitting previous footprints");
            _this.enqueueAndSubmit(prevExecution);
        });
        this.stateTracker.on(state_tracker_1.StateTracker.EXECUTION_ID_CHANGED, function (prevExecution, newExecutionId) {
            _this.logger.warn("Execution id changed to'" + newExecutionId + "'. Collecting previous footprints");
            _this.enqueueCurrentFootprints(prevExecution.executionId, null, prevExecution.buildSessionId);
        });
        this.stateTracker.on(state_tracker_1.StateTracker.TEST_STAGE_CHANGED, function (prevExecution, newTestStage) {
            _this.logger.warn("Execution points to different test stage '" + newTestStage + "'. Collecting and submitting previous footprints");
            _this.enqueueAndSubmit(prevExecution);
        });
    };
    FootprintsProcess.prototype.enqueueAndSubmit = function (executionData, isFinalFootprints) {
        if (isFinalFootprints === void 0) { isFinalFootprints = false; }
        this.enqueueCurrentFootprints(executionData.executionId, null, executionData.buildSessionId, isFinalFootprints);
        this.submitQueuedFootprints();
    };
    FootprintsProcess.prototype.flushCurrentFootprints = function (isFinalFootprints) {
        if (isFinalFootprints === void 0) { isFinalFootprints = false; }
        var currentTestIdentifier = this.stateTracker.getCurrentTestIdentifier();
        var testIdentifierParts = state_tracker_1.StateTracker.splitTestIdToExecutionAndTestName(currentTestIdentifier);
        if (currentTestIdentifier != null) {
            if (!this.stateTracker.shouldCollectHits() && this.stateTracker.isAnonymousColor(currentTestIdentifier)) {
                this.logger.info("Enqueue footprints interval - couldn't find active execution for anonymous footprints. skip enqueue footprints process. currentTestIdentifier: '%s'", currentTestIdentifier);
                this.handleNoExecutionFound();
                return;
            }
            this.logger.debug("Enqueue footprints interval - start enqueuing process. currentTestIdentifier: '%s'", currentTestIdentifier);
            this.enqueueCurrentFootprints(this.stateTracker.currentExecution.executionId, testIdentifierParts.testName, this.stateTracker.currentExecution.buildSessionId, isFinalFootprints);
        }
        else {
            this.logger.info("Enqueue footprints interval - no test identifier. skip enqueuing process. ");
        }
    };
    FootprintsProcess.prototype.handleNoExecutionFound = function () {
        if (this.stateTracker.openExecutionFoundOnce) {
            this.logger.debug('No active execution and not in initFootprints mode, updating coverage snapshot');
            this.hitsCollector.dropHits();
        }
    };
    FootprintsProcess.prototype.isInitFootprints = function () {
        return this.stateTracker.openExecutionFoundOnce && !this.footprintsEnqueueOnce;
    };
    FootprintsProcess.prototype.hasOngoingRequests = function () {
        return this.ongoingRequestsCounter > 0;
    };
    FootprintsProcess.ALARM_FIRED = "alarm";
    return FootprintsProcess;
}());
exports.FootprintsProcess = FootprintsProcess;
//# sourceMappingURL=index.js.map