"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var source_maps_utils_1 = require("../source-maps-utils");
var source_map_handler_1 = require("../source-map-handler");
var SourceMapData = /** @class */ (function () {
    function SourceMapData(logger) {
        this.logger = logger;
        this.pathToConsumer = new Map();
    }
    SourceMapData.prototype.getSourcePosition = function (relativePath, absolutePath, start, projectRoot) {
        var sourceMapForModule = this.getSourceMapForModule(relativePath);
        if (sourceMapForModule) {
            var startPositionArr = [start.line, start.column];
            var handler = new source_map_handler_1.SourceMapHandler(sourceMapForModule, absolutePath, startPositionArr, [], projectRoot, this.logger);
            handler.resolvePaths();
            if (!handler.shouldSkipped) {
                return { path: handler.relativePath, start: { line: handler.start.line, column: handler.start.column } };
            }
        }
        return null;
    };
    SourceMapData.prototype.getSourceMapForModule = function (relPath) {
        if (!this.pathToConsumer.get(relPath)) {
            this.pathToConsumer.set(relPath, source_maps_utils_1.SourceMapsUtils.readSourceMaps(relPath));
            this.logger.debug("Read source maps for file '" + relPath + "'");
        }
        return this.pathToConsumer.get(relPath);
    };
    return SourceMapData;
}());
exports.SourceMapData = SourceMapData;
//# sourceMappingURL=source-map-data.js.map