import { HttpClientConfigData, BuildSessionData, CreateBuildSessionIdResponse, GetVersionRequest, IBuildEndData, IColoredExecutionRequest, IHttpClient, ITestsRecommendationResponse, StartExecutionRequest, SubmitLogsRequest, UploadReportsBody, BuildMappingRequest, GetRemoteConfigRequest, AgentData, EndExecutionRequest } from "./contracts";
import { IEventsFile } from "../events-process/events-contracts";
import { IAgentEventRequest } from "../agent-events/agent-events-conracts";
import { AgentInstanceData } from "../agent-instance-data";
import { IExecutionResponse, IIntegrationBuildDependency } from "../contracts";
import { IFootprintsV6File } from "../footprints-process-v6/contracts";
import bunyan = require('bunyan');
export declare class BackendProxy {
    static readonly DEFAULT_HTTP_MAX_ATTEMPTS = 6;
    static readonly DEFAULT_HTTP_ATTEMPT_INTERVAL: number;
    protected config: HttpClientConfigData;
    private client;
    protected logger: bunyan;
    protected agentInstanceData: AgentInstanceData;
    constructor(agentInstanceData: AgentInstanceData, config: HttpClientConfigData, logger: bunyan, httpClient?: IHttpClient);
    getBuildSession(buildSessionId: string, callback: (err: Error, response: BuildSessionData) => void): void;
    createBuildSessionId(request: BuildSessionData, callback: (err: Error, response: CreateBuildSessionIdResponse) => void): void;
    createBuildSessionIdPromise<T extends BuildSessionData>(request: T): Promise<string>;
    getRecommendedVersion(request: GetVersionRequest): Promise<any>;
    submitBuildMapping(request: BuildMappingRequest): Promise<any>;
    submitLogs(request: SubmitLogsRequest, callback: (err: Error, body: any) => void): void;
    getRemoteConfig(request: GetRemoteConfigRequest, callback: (err: Error, body: any) => void): void;
    getRemoteConfigPromise(request: GetRemoteConfigRequest): Promise<unknown>;
    startExecution(request: StartExecutionRequest, callback: (err: Error, body: any) => void): void;
    startColoredExecution(request: IColoredExecutionRequest): Promise<void>;
    testExecutionV4(labId: string, async?: boolean, executionId?: string): Promise<IExecutionResponse>;
    uploadReport(request: UploadReportsBody | AgentData, callback: (err: Error, body: any) => void): void;
    externalReport(request: UploadReportsBody | AgentData, callback: (err: Error, body: any) => void): void;
    endExecution(request: EndExecutionRequest, callback: (err: Error, body: any) => void): void;
    submitEvents(packetToSend: IEventsFile, callback: (err: Error, body: any) => void, async?: boolean): void;
    submitEventsPromise(packetToSend: IEventsFile): Promise<void>;
    submitBlob(body: string, buildSessionId: string, blobId: string, callback: (err: Error, body: any) => void): void;
    submitAgentEvent<T>(body: IAgentEventRequest<T>): Promise<void>;
    getTestsRecommendation(buildSessionId: string, stage: string): Promise<ITestsRecommendationResponse>;
    addOrUpdateIntegrationBuildComponents(buildSessionId: string, components: IIntegrationBuildDependency[], agentId: string): Promise<unknown>;
    deleteIntegrationBuildComponents(buildSessionId: string, components: IIntegrationBuildDependency[], agentId: string): Promise<unknown>;
    buildEnd(data: IBuildEndData): Promise<void>;
    submitFootprintsV6(footprintsPacket: IFootprintsV6File, executionBsid: string, testStage: string, buildSessionId: string): Promise<void>;
    protected invokeCallback(callback: (err: Error, apiResponse: any) => void, err: Error, body: any, map?: (body: any) => any): void;
    private submitPostRequestWithRetries;
    private submitPutRequestWithRetries;
    private submitDelRequestWithRetries;
    private submitGetRequestWithRetries;
    private makeRequestWithRetries;
    private shouldRetryRequest;
}
