import request = require('request');
import { IBuildSessionParams, IEventResponse, IRetryCounters, HttpVerb, IRoute, IAgentEventsRequest, IBuildEndData } from "./contracts";
import { RequestResponse } from "request";
import { IExecutionData } from "../../../contracts";
/**
 * A proxy for interacting sealights mock server using promises
 */
export declare class MockServerProxy {
    static readonly SERVER_URL = "http://localhost:9900";
    private static readonly GET_ALL_FOOTPRINTS;
    private static readonly GET_BUILD_MAPPING;
    private static readonly RESET;
    private static readonly GET_TOKEN;
    private static readonly CREATE_BUILDSESSION_ID;
    private static readonly SET_RECOMMENDED_VERSION;
    private static readonly SET_ACTIVE_BSID;
    private static readonly GET_TEST_EVENTS;
    private static readonly SET_TEST_EXCLUSIONS;
    private static readonly GET_RETRY_COUNTERS;
    private static readonly GET_AGENT_EVENTS;
    private static readonly SETTINGS;
    static readonly TEST_EXCLUSION = "/api/v2/test-exclusions/:buildSessionId/:testStage";
    static readonly GET_BUILD_END = "/api/test/buildend";
    static readonly GET_LOGS = "/api/test/logs";
    static readonly SET_DELAY = "/api/test/setDelay";
    static readonly SET_ENABLED = "/api/test/setEnabled";
    static readonly SET_BUILD_END = "/api/v3/agents/buildend";
    static readonly SET_TEST_EXECUTION = "/api/test/testExecution";
    static readonly FOOTPRINTS_V6 = "/api/v6/agents/:executionBsId/footprints/:testStage/:buildSessionId";
    static getAllFootprints(): Promise<any[]>;
    static setRecommendedVersion(requestParams: any): Promise<void>;
    static createBuildSessionId(buildSessionParams: IBuildSessionParams): Promise<string>;
    static getBuildMapping(): Promise<any[]>;
    static resetState(): Promise<void>;
    static getToken(): Promise<string>;
    static setActiveBSID(buildSessionId: string): Promise<string>;
    static getTestEvents(): Promise<IEventResponse[]>;
    static setTestExclusions(buildSessionId: string, testStage: string, testRecommendations: any): Promise<any>;
    static getRetryCounters(): Promise<IRetryCounters>;
    static getAgentEvents(): Promise<IAgentEventsRequest[]>;
    static getBuildEndStatus(): Promise<IBuildEndData>;
    static getLogs(): Promise<IBuildEndData>;
    static setDelay(duration_ms: number, requestToBeDelayed: string): Promise<request.RequestResponse>;
    static setEnabled(enabled: boolean): Promise<request.RequestResponse>;
    static setRouteSettings(url: string, verb: HttpVerb, status: number, requestsBeforeStatusExpires: number): Promise<void>;
    static setTestExecution(execution: {
        execution: IExecutionData;
    }): Promise<void>;
    static getSettings(): Promise<IRoute[]>;
    private static executeSlRequest;
    static executeRequest(url: string, verb: HttpVerb, shouldParse?: boolean, requestData?: any): Promise<RequestResponse>;
}
